@@ -1,6 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2019 Jochen Ulrich <jochenulrich@t-online.de>
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
@@ -45,6 +46,7 @@
 #include <tools/error.h>
 
 #include <QtCore/qobject.h>
+#include <QtCore/qtimer.h>
 
 namespace qbs {
 class ErrorInfo;
@@ -58,23 +60,23 @@ class AbstractCommandExecutor : public QObject
 {
     Q_OBJECT
 public:
-    explicit AbstractCommandExecutor(const Internal::Logger &logger, QObject *parent = 0);
+    explicit AbstractCommandExecutor(Internal::Logger logger, QObject *parent = nullptr);
 
     void setMainThreadScriptEngine(ScriptEngine *engine) { m_mainThreadScriptEngine = engine; }
     void setDryRunEnabled(bool enabled) { m_dryRun = enabled; }
     void setEchoMode(CommandEchoMode echoMode) { m_echoMode = echoMode; }
 
-    virtual void cancel() = 0;
+    virtual void cancel(const qbs::ErrorInfo &reason = {}) = 0;
 
-    void start(Transformer *transformer, const AbstractCommand *cmd);
+    void start(Transformer *transformer, AbstractCommand *cmd);
 
 signals:
     void reportCommandDescription(const QString &highlight, const QString &message);
     void finished(const qbs::ErrorInfo &err = ErrorInfo()); // !hasError() <=> command successful
 
 protected:
-    virtual void doReportCommandDescription();
-    const AbstractCommand *command() const { return m_command; }
+    virtual void doReportCommandDescription(const QString &productName);
+    AbstractCommand *command() const { return m_command; }
     Transformer *transformer() const { return m_transformer; }
     ScriptEngine *scriptEngine() const { return m_mainThreadScriptEngine; }
     bool dryRun() const { return m_dryRun; }
@@ -83,14 +85,17 @@ protected:
 
 private:
     virtual void doSetup() { };
-    virtual void doStart() = 0;
+    virtual bool doStart() = 0;
+
+    void startTimeout();
 
 private:
-    const AbstractCommand *m_command;
+    AbstractCommand *m_command;
     Transformer *m_transformer;
     ScriptEngine *m_mainThreadScriptEngine;
     bool m_dryRun;
     Internal::Logger m_logger;
+    QTimer m_watchdog;
 };
 
 } // namespace Internal
