@@ -56,13 +56,9 @@
 namespace qbs {
 namespace Internal {
 
-RuleNode::RuleNode()
-{
-}
+RuleNode::RuleNode() = default;
 
-RuleNode::~RuleNode()
-{
-}
+RuleNode::~RuleNode() = default;
 
 void RuleNode::accept(BuildGraphVisitor *visitor)
 {
@@ -78,21 +74,19 @@ QString RuleNode::toString() const
             + QLatin1String(" located at ") + m_rule->prepareScript.location().toString();
 }
 
-void RuleNode::apply(const Logger &logger,
-                     const std::unordered_map<QString, const ResolvedProduct *> &productsByName,
-                     const std::unordered_map<QString, const ResolvedProject *> &projectsByName,
-                     ApplicationResult *result)
+RuleNode::ApplicationResult RuleNode::apply(
+    const Logger &logger,
+    const std::unordered_map<QString, const ResolvedProduct *> &productsByName,
+    const std::unordered_map<QString, const ResolvedProject *> &projectsByName)
 {
+    ApplicationResult result;
     ArtifactSet allCompatibleInputs = currentInputArtifacts();
-    const ArtifactSet explicitlyDependsOn
-            = RulesApplicator::collectExplicitlyDependsOn(m_rule.get(), product.get());
-    const ArtifactSet auxiliaryInputs
-            = RulesApplicator::collectAuxiliaryInputs(m_rule.get(), product.get());
+    const ArtifactSet explicitlyDependsOn = RulesApplicator::collectExplicitlyDependsOn(
+        m_rule.get(), product.get());
     const ArtifactSet addedInputs = allCompatibleInputs - m_oldInputArtifacts;
     const ArtifactSet removedInputs = m_oldInputArtifacts - allCompatibleInputs;
-    const ArtifactSet changedInputs = changedInputArtifacts(allCompatibleInputs,
-                                                            explicitlyDependsOn,
-                                                            auxiliaryInputs);
+    const ArtifactSet changedInputs = changedInputArtifacts(
+        allCompatibleInputs, explicitlyDependsOn);
     bool upToDate = changedInputs.empty() && addedInputs.empty() && removedInputs.empty();
 
     qCDebug(lcBuildGraph).noquote().nospace()
@@ -149,7 +143,7 @@ void RuleNode::apply(const Logger &logger,
 
     if (upToDate) {
         qCDebug(lcExec) << "rule is up to date. Skipping.";
-        return;
+        return result;
     }
 
     const bool mustApplyRule = !inputs.empty() || !m_rule->declaresInputs()
@@ -175,29 +169,33 @@ void RuleNode::apply(const Logger &logger,
             // parent must always have a transformer, because it's generated.
             QBS_CHECK(parent->transformer);
 
-            // artifact is a former input of m_rule and parent was created by m_rule
-            // the inputs of the transformer must contain artifact
-            QBS_CHECK(parent->transformer->inputs.contains(artifact));
+            // Artifact is a former input of m_rule and parent was created by m_rule.
+            // The inputs of the transformer must contain artifact, or it is a scanned dependency,
+            // in which case it is not relevant here.
+            if (!parent->transformer->inputs.contains(artifact)) {
+                QBS_CHECK(parent->childrenAddedByScanner.contains(artifact));
+                continue;
+            }
 
             if (removedInputForcesOutputRemoval)
                 outputArtifactsToRemove += parent;
             else
-                connectionsToBreak.push_back(std::make_pair(parent, artifact));
+                connectionsToBreak.emplace_back(parent, artifact);
         }
         disconnect(this, artifact);
     }
     for (const auto &connection : connectionsToBreak)
         disconnect(connection.first, connection.second);
     if (!outputArtifactsToRemove.empty()) {
-        RulesApplicator::handleRemovedRuleOutputs(inputs, outputArtifactsToRemove,
-                                                  result->removedArtifacts, logger);
+        RulesApplicator::handleRemovedRuleOutputs(
+            inputs, outputArtifactsToRemove, result.removedArtifacts, logger);
     }
 
     if (mustApplyRule) {
         RulesApplicator applicator(product.lock(), productsByName, projectsByName, logger);
         applicator.applyRule(this, inputs, explicitlyDependsOn);
-        result->createdArtifacts = applicator.createdArtifacts();
-        result->invalidatedArtifacts = applicator.invalidatedArtifacts();
+        result.createdArtifacts = applicator.createdArtifacts();
+        result.invalidatedArtifacts = applicator.invalidatedArtifacts();
         m_lastApplicationTime = FileTime::currentTime();
         if (applicator.ruleUsesIo())
             m_needsToConsiderChangedInputs = true;
@@ -206,8 +204,8 @@ void RuleNode::apply(const Logger &logger,
     }
     m_oldInputArtifacts = allCompatibleInputs;
     m_oldExplicitlyDependsOn = explicitlyDependsOn;
-    m_oldAuxiliaryInputs = auxiliaryInputs;
     product->topLevelProject()->buildData->setDirty();
+    return result;
 }
 
 void RuleNode::load(PersistentPool &pool)
@@ -227,13 +225,13 @@ int RuleNode::transformerCount() const
     Set<const Transformer *> transformers;
     for (const Artifact * const output : filterByType<Artifact>(parents))
         transformers.insert(output->transformer.get());
-    return transformers.size();
+    return int(transformers.size());
 }
 
 ArtifactSet RuleNode::currentInputArtifacts() const
 {
     ArtifactSet s;
-    for (const FileTag &t : qAsConst(m_rule->inputs)) {
+    for (const FileTag &t : std::as_const(m_rule->inputs)) {
         for (Artifact *artifact : product->lookupArtifactsByFileTag(t)) {
             if (artifact->isTargetOfModule())
                 continue;
@@ -250,7 +248,7 @@ ArtifactSet RuleNode::currentInputArtifacts() const
 
     if (m_rule->inputsFromDependencies.empty())
         return s;
-    for (const FileTag &t : qAsConst(m_rule->inputsFromDependencies)) {
+    for (const FileTag &t : std::as_const(m_rule->inputsFromDependencies)) {
         for (Artifact *artifact : product->lookupArtifactsByFileTag(t)) {
             if (!artifact->isTargetOfModule())
                 continue;
@@ -262,7 +260,7 @@ ArtifactSet RuleNode::currentInputArtifacts() const
         }
     }
 
-    for (const ResolvedProductConstPtr &dep : qAsConst(product->dependencies)) {
+    for (const auto &dep : std::as_const(product->dependencies)) {
         if (!dep->buildData)
             continue;
         for (Artifact * const a : filterByType<Artifact>(dep->buildData->allNodes())) {
@@ -275,9 +273,8 @@ ArtifactSet RuleNode::currentInputArtifacts() const
     return s;
 }
 
-ArtifactSet RuleNode::changedInputArtifacts(const ArtifactSet &allCompatibleInputs,
-                                            const ArtifactSet &explicitlyDependsOn,
-                                            const ArtifactSet &auxiliaryInputs) const
+ArtifactSet RuleNode::changedInputArtifacts(
+    const ArtifactSet &allCompatibleInputs, const ArtifactSet &explicitlyDependsOn) const
 {
     ArtifactSet changedInputArtifacts;
     if (explicitlyDependsOn != m_oldExplicitlyDependsOn)
@@ -289,12 +286,6 @@ ArtifactSet RuleNode::changedInputArtifacts(const ArtifactSet &allCompatibleInpu
         if (artifact->timestamp() > m_lastApplicationTime)
             return allCompatibleInputs;
     }
-    if (auxiliaryInputs != m_oldAuxiliaryInputs)
-        return allCompatibleInputs;
-    for (Artifact * const artifact : auxiliaryInputs) {
-        if (artifact->timestamp() > m_lastApplicationTime)
-            return allCompatibleInputs;
-    }
     for (Artifact * const artifact : allCompatibleInputs) {
         if (artifact->timestamp() > m_lastApplicationTime)
             changedInputArtifacts.insert(artifact);
@@ -314,11 +305,6 @@ void RuleNode::removeOldInputArtifact(Artifact *artifact)
                               << "from rule" << rule()->toString();
         m_oldExplicitlyDependsOn.insert(nullptr);
     }
-    if (m_oldAuxiliaryInputs.remove(artifact)) {
-        qCDebug(lcBuildGraph) << "remove old auxiliaryInput" << artifact->filePath()
-                              << "from rule" << rule()->toString();
-        m_oldAuxiliaryInputs.insert(nullptr);
-    }
 }
 
 } // namespace Internal
