@@ -31,6 +31,8 @@
 
 #include "tst_blackboxbase.h"
 
+#include <optional>
+
 namespace qbs {
 class Version;
 } // namespace qbs
@@ -66,14 +68,17 @@ private slots:
     void iconset();
     void iconsetApp();
     void infoPlist();
+    void privacymanifest();
     void infoPlistVariables();
+    void lipoSymlinks();
+    void lipoSymlinks_data();
     void objcArc();
     void overrideInfoPlist();
     void xcode();
 
 private:
-    QVariantMap findXcode(int *status = nullptr);
-    qbs::Version findXcodeVersion();
+    std::optional<QVariantMap> findXcode(int *status = nullptr);
+    std::optional<qbs::Version> findXcodeVersion();
 };
 
 #endif // TST_BLACKBOXAPPLE_H
