@@ -33,12 +33,13 @@
 #include <tools/preferences.h>
 #include <tools/profile.h>
 
+#include <QtCore/qjsonarray.h>
 #include <QtCore/qjsondocument.h>
+#include <QtCore/qjsonobject.h>
 
 #define WAIT_FOR_NEW_TIMESTAMP() waitForNewTimestamp(testDataDir)
 
 using qbs::Internal::HostOsInfo;
-using qbs::Profile;
 
 TestBlackboxQt::TestBlackboxQt() : TestBlackboxBase (SRCDIR "/testdata-qt", "blackbox-qt")
 {
@@ -131,6 +132,28 @@ void TestBlackboxQt::dbusInterfaces()
     QCOMPARE(runQbs(), 0);
 }
 
+void TestBlackboxQt::emscriptenHtml()
+{
+    QDir::setCurrent(testDataDir + "/emscripten-html");
+    QCOMPARE(runQbs(), 0);
+    if (m_qbsStdout.contains("is emscripten: false"))
+        QSKIP("Skipping emscripten test");
+    QVERIFY(m_qbsStdout.contains("is emscripten: true"));
+
+    const auto relativeInstallRoot = relativeBuildDir() + QStringLiteral("/install-root/");
+    QVERIFY(!regularFileExists(relativeInstallRoot + QStringLiteral("qtloader.js")));
+    QVERIFY(!regularFileExists(relativeInstallRoot + QStringLiteral("qtlogo.svg")));
+    QVERIFY(!regularFileExists(relativeInstallRoot + QStringLiteral("app.html")));
+
+    const QStringList params = {QStringLiteral("products.app.generateHtml:true")};
+    QCOMPARE(runQbs(QbsRunParameters("resolve", params)), 0);
+    QCOMPARE(runQbs(QbsRunParameters("build", params)), 0);
+
+    QVERIFY(regularFileExists(relativeInstallRoot + QStringLiteral("qtloader.js")));
+    QVERIFY(regularFileExists(relativeInstallRoot + QStringLiteral("qtlogo.svg")));
+    QVERIFY(regularFileExists(relativeInstallRoot + QStringLiteral("app.html")));
+}
+
 void TestBlackboxQt::forcedMoc()
 {
     QDir::setCurrent(testDataDir + "/forced-moc");
@@ -150,12 +173,26 @@ void TestBlackboxQt::includedMocCpp()
     if (m_qbsStdout.contains("using qt4"))
         QSKIP("Qt version too old");
     QVERIFY2(!m_qbsStdout.contains("compiling moc_myobject.cpp"), m_qbsStdout.constData());
+
     WAIT_FOR_NEW_TIMESTAMP();
     REPLACE_IN_FILE("myobject.cpp", "#include <moc_myobject.cpp", "// #include <moc_myobject.cpp");
+    QbsRunParameters failParams;
+    failParams.expectFailure = true;
+    QEXPECT_FAIL(nullptr, "not worth supporting", Continue);
+    QCOMPARE(runQbs(failParams), 0);
+
+    WAIT_FOR_NEW_TIMESTAMP();
+    touch("myobject.h");
     QCOMPARE(runQbs(), 0);
     QVERIFY2(m_qbsStdout.contains("compiling moc_myobject.cpp"), m_qbsStdout.constData());
+
     WAIT_FOR_NEW_TIMESTAMP();
     REPLACE_IN_FILE("myobject.cpp", "// #include <moc_myobject.cpp", "#include <moc_myobject.cpp");
+    QEXPECT_FAIL(nullptr, "not worth supporting", Continue);
+    QCOMPARE(runQbs(failParams), 0);
+
+    WAIT_FOR_NEW_TIMESTAMP();
+    touch("myobject.h");
     QCOMPARE(runQbs(), 0);
     QVERIFY2(!m_qbsStdout.contains("compiling moc_myobject.cpp"), m_qbsStdout.constData());
 }
@@ -174,6 +211,9 @@ void TestBlackboxQt::lrelease()
 {
     QDir::setCurrent(testDataDir + QLatin1String("/lrelease"));
     QCOMPARE(runQbs(), 0);
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
+
     QVERIFY(regularFileExists(relativeProductBuildDir("lrelease-test") + "/de.qm"));
     QVERIFY(regularFileExists(relativeProductBuildDir("lrelease-test") + "/hu.qm"));
 
@@ -200,6 +240,127 @@ void TestBlackboxQt::lrelease()
     QVERIFY(!regularFileExists(relativeProductBuildDir("lrelease-test") + "/hu.qm"));
 }
 
+void TestBlackboxQt::lupdate()
+{
+    QDir::setCurrent(testDataDir + "/lupdate");
+
+    // Initial run.
+    QCOMPARE(runQbs(QbsRunParameters(QStringList{"-p", "lupdate-runner"})), 0);
+    QVERIFY2(m_qbsStderr.isEmpty(), m_qbsStderr.constData());
+    QVERIFY2(m_qbsStdout.contains("creating lupdate project file"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("updating translation files"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("Found 4 source text"), m_qbsStdout.constData());
+    QFile jsonFile(relativeProductBuildDir("lupdate-runner") + "/lupdate.json");
+    QVERIFY2(jsonFile.open(QIODevice::ReadOnly), qPrintable(jsonFile.fileName()));
+    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonFile.readAll());
+    QVERIFY(jsonDoc.isObject());
+    QJsonObject mainObj = jsonDoc.object();
+    QJsonArray tsFiles = mainObj.value("translations").toArray();
+    QCOMPARE(tsFiles.size(), 1);
+    QJsonArray mainSources = mainObj.value("sources").toArray();
+    QCOMPARE(mainSources.size(), 1);
+    QVERIFY(mainSources.at(0).toString().endsWith(".ui"));
+    QJsonArray subProjects = mainObj.value("subProjects").toArray();
+    QCOMPARE(subProjects.size(), 3);
+    for (const QJsonValue &v : subProjects) {
+        const QJsonObject subProject = v.toObject();
+        const QString name = subProject.value("projectFile").toString();
+        QVERIFY2(name == "cxx 1" || name == "cxx 2" || name == "cxx 3", qPrintable(name));
+        const QJsonArray includePaths = subProject.value("includePaths").toArray();
+        bool hasSubdir = false;
+        bool hasP1Headers = false;
+        bool hasP2Headers = false;
+        for (const QJsonValue &v : includePaths) {
+            const QString vString = v.toString();
+            if (vString.contains("subdir"))
+                hasSubdir = true;
+            else if (vString.contains("qt1."))
+                hasP1Headers = true;
+            else if (vString.contains("qt2."))
+                hasP2Headers = true;
+        }
+        QVERIFY(hasP1Headers != hasP2Headers);
+        if (hasSubdir) {
+            QVERIFY(hasP1Headers);
+            QVERIFY(!hasP2Headers);
+        }
+        const QJsonArray sources = subProject.value("sources").toArray();
+        const int expectedSourceCount = hasSubdir ? 1 : 3;
+        if (sources.size() != expectedSourceCount)
+            qDebug() << sources;
+        QCOMPARE(sources.size(), expectedSourceCount);
+    }
+    jsonFile.close();
+
+    // Run again. The JSON file should not get updated.
+    QCOMPARE(runQbs(QbsRunParameters(QStringList{"-p", "lupdate-runner"})), 0);
+    QVERIFY2(m_qbsStderr.isEmpty(), m_qbsStderr.constData());
+    QVERIFY2(!m_qbsStdout.contains("creating lupdate project file"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("updating translation files"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("Found 4 source text"), m_qbsStdout.constData());
+
+    // Add source file, verify that the JSON file gets updated.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("lupdate.qbs", "/*\"qt2-new.cpp\"*/", "\"qt2-new.cpp\"");
+    QCOMPARE(runQbs(QbsRunParameters(QStringList{"-p", "lupdate-runner"})), 0);
+    QVERIFY2(m_qbsStderr.isEmpty(), m_qbsStderr.constData());
+    QVERIFY2(m_qbsStdout.contains("creating lupdate project file"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("updating translation files"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("Found 5 source text"), m_qbsStdout.constData());
+
+    // Change include paths, verify that the JSON file gets updated and the sources re-grouped.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE(
+        "lupdate.qbs", "/*cpp.includePaths: \"subdir\"*/", "cpp.includePaths: \"subdir\"");
+    REPLACE_IN_FILE("lupdate.qbs", "cpp.includePaths: outer.concat(\"subdir\")", "");
+    QCOMPARE(runQbs(QbsRunParameters(QStringList{"-p", "lupdate-runner"})), 0);
+    QVERIFY2(m_qbsStderr.isEmpty(), m_qbsStderr.constData());
+    QVERIFY2(m_qbsStdout.contains("creating lupdate project file"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("updating translation files"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("Found 5 source text"), m_qbsStdout.constData());
+    QVERIFY2(jsonFile.open(QIODevice::ReadOnly), qPrintable(jsonFile.fileName()));
+    jsonDoc = QJsonDocument::fromJson(jsonFile.readAll());
+    QVERIFY(jsonDoc.isObject());
+    mainObj = jsonDoc.object();
+    tsFiles = mainObj.value("translations").toArray();
+    QCOMPARE(tsFiles.size(), 1);
+    mainSources = mainObj.value("sources").toArray();
+    QCOMPARE(mainSources.size(), 1);
+    QVERIFY(mainSources.at(0).toString().endsWith(".ui"));
+    subProjects = mainObj.value("subProjects").toArray();
+    QCOMPARE(subProjects.size(), 2);
+    for (const QJsonValue &v : subProjects) {
+        const QJsonObject subProject = v.toObject();
+        const QString name = subProject.value("projectFile").toString();
+        QVERIFY2(name == "cxx 1" || name == "cxx 2", qPrintable(name));
+        const QJsonArray includePaths = subProject.value("includePaths").toArray();
+        const QJsonArray sources = subProject.value("sources").toArray();
+        const int expectedSourceCount = 4;
+        if (sources.size() != expectedSourceCount)
+            qDebug() << sources;
+        QCOMPARE(sources.size(), expectedSourceCount);
+    }
+
+    // Change an unrelated property. The JSON file should not get updated.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("lupdate.qbs", "// cpp.cxxLanguageVersion:", "cpp.cxxLanguageVersion:");
+    QCOMPARE(runQbs(QbsRunParameters(QStringList{"-p", "lupdate-runner"})), 0);
+    QVERIFY2(m_qbsStderr.isEmpty(), m_qbsStderr.constData());
+    QVERIFY2(m_qbsStdout.contains("Resolving"), m_qbsStdout.constData());
+    QVERIFY2(!m_qbsStdout.contains("creating lupdate project file"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("updating translation files"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("Found 5 source text"), m_qbsStdout.constData());
+
+    // Change a source file. The JSON file should not get updated.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("qt2-main.cpp", "// auto s2", "auto s2");
+    QCOMPARE(runQbs(QbsRunParameters(QStringList{"-p", "lupdate-runner"})), 0);
+    QVERIFY2(m_qbsStderr.isEmpty(), m_qbsStderr.constData());
+    QVERIFY2(!m_qbsStdout.contains("creating lupdate project file"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("updating translation files"), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("Found 6 source text"), m_qbsStdout.constData());
+}
+
 void TestBlackboxQt::metaTypes_data()
 {
     QTest::addColumn<bool>("generate");
@@ -239,18 +400,15 @@ void TestBlackboxQt::metaTypes()
 void TestBlackboxQt::mixedBuildVariants()
 {
     QDir::setCurrent(testDataDir + "/mixed-build-variants");
-    const SettingsPtr s = settings();
-    Profile profile(profileName(), s.get());
-    if (profileToolchain(profile).contains("msvc")) {
-        QbsRunParameters params;
-        params.arguments << "qbs.buildVariant:debug";
-        params.expectFailure = true;
-        QVERIFY(runQbs(params) != 0);
+    QbsRunParameters params;
+    params.expectFailure = true;
+    QVERIFY(runQbs(params) != 0);
+    const bool isMsvc = m_qbsStdout.contains("is msvc: true");
+    const bool isNotMsvc = m_qbsStdout.contains("is msvc: false");
+    if (isMsvc) {
         QVERIFY2(m_qbsStderr.contains("not allowed"), m_qbsStderr.constData());
     } else {
-        QbsRunParameters params;
-        params.expectFailure = true;
-        QVERIFY(runQbs(params) != 0);
+        QVERIFY(isNotMsvc);
         QVERIFY2(m_qbsStderr.contains("not supported"), m_qbsStderr.constData());
     }
 }
@@ -285,6 +443,28 @@ void TestBlackboxQt::mocSameFileName()
     QCOMPARE(m_qbsStdout.count("compiling moc_someclass.cpp"), 2);
 }
 
+void TestBlackboxQt::noMocRunAfterTouchingOtherCppFile()
+{
+    QDir::setCurrent(testDataDir + "/no-moc-run-after-touching-other-cpp-file");
+    QCOMPARE(runQbs(), 0);
+    QVERIFY(m_qbsStdout.contains("moc header1.h"));
+    QVERIFY(m_qbsStdout.contains("moc header2.h"));
+    QVERIFY(m_qbsStdout.contains("compiling moc_header1.cpp"));
+    QVERIFY(m_qbsStdout.contains("compiling moc_header2.cpp"));
+    QVERIFY(m_qbsStdout.contains("compiling main.cpp"));
+    QVERIFY(m_qbsStdout.contains("linking"));
+
+    WAIT_FOR_NEW_TIMESTAMP();
+    touch("main.cpp");
+    QCOMPARE(runQbs(), 0);
+    QVERIFY(!m_qbsStdout.contains("moc header1.h"));
+    QVERIFY(!m_qbsStdout.contains("moc header2.h"));
+    QVERIFY(!m_qbsStdout.contains("compiling moc_header1.cpp"));
+    QVERIFY(!m_qbsStdout.contains("compiling moc_header2.cpp"));
+    QVERIFY(m_qbsStdout.contains("compiling main.cpp"));
+    QVERIFY(m_qbsStdout.contains("linking"));
+}
+
 void TestBlackboxQt::noRelinkOnQDebug()
 {
     QFETCH(QString, checkMode);
@@ -295,6 +475,9 @@ void TestBlackboxQt::noRelinkOnQDebug()
 
     // Target check.
     QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("is emscripten: true"))
+        QSKIP("Irrelevant for emscripten");
+    QVERIFY(m_qbsStdout.contains("is emscripten: false"));
     QVERIFY2(m_qbsStdout.contains("is GCC: "), m_qbsStdout.constData());
     QVERIFY2(m_qbsStdout.contains("is MinGW: "), m_qbsStdout.constData());
     QVERIFY2(m_qbsStdout.contains("is Darwin: "), m_qbsStdout.constData());
@@ -345,8 +528,6 @@ void TestBlackboxQt::pkgconfig()
     QbsRunParameters params;
     params.command = "run";
     QCOMPARE(runQbs(params), 0);
-    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
-        QSKIP("pkgconfig or Qt not found");
 }
 
 void TestBlackboxQt::pkgconfigQt()
@@ -397,6 +578,20 @@ void TestBlackboxQt::pkgconfigQt_data()
             << QStringList({"moduleProviders.qbspkgconfig.sysroot:/some/fake/sysroot"}) << false;
 }
 
+void TestBlackboxQt::pkgconfigNoQt()
+{
+    QDir::setCurrent(testDataDir + "/pkgconfig-qt");
+    rmDirR(relativeBuildDir());
+    QbsRunParameters params("build", {"-f", "pkgconfig-qt.qbs"});
+    params.arguments << "moduleProviders.qbspkgconfig.libDirs:nonexistent";
+    params.expectFailure = true;
+
+    QCOMPARE(runQbs(params) == 0, false);
+    QVERIFY2(m_qbsStderr.contains("Dependency 'Qt.core' not found for product 'p'"), m_qbsStderr);
+    // QBS-1777: basic check for JS exceptions in case of missing Qt
+    QVERIFY2(!m_qbsStderr.contains("Error executing provider for module 'Qt.core'"), m_qbsStderr);
+}
+
 void TestBlackboxQt::pluginMetaData()
 {
     QDir::setCurrent(testDataDir + "/plugin-meta-data");
@@ -474,6 +669,8 @@ void TestBlackboxQt::qdoc()
 {
     QDir::setCurrent(testDataDir + "/qdoc");
     QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
     if (m_qbsStdout.contains("Qt is too old"))
         QSKIP("Skip test since qdoc3 does not work properly");
     QCOMPARE(runQbs(), 0);
@@ -484,18 +681,24 @@ void TestBlackboxQt::qmlDebugging()
 {
     QDir::setCurrent(testDataDir + "/qml-debugging");
     QCOMPARE(runQbs(), 0);
-    const SettingsPtr s = settings();
-    Profile profile(profileName(), s.get());
-    if (!profileToolchain(profile).contains("gcc"))
-        return;
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
+
+    const bool isGcc = m_qbsStdout.contains("is gcc: true");
+    const bool isNotGcc = m_qbsStdout.contains("is gcc: false");
+    if (isNotGcc)
+        QSKIP("The remainder of this test only applies to gcc");
+    QVERIFY(isGcc);
+
     QProcess nm;
-    nm.start("nm", QStringList(relativeExecutableFilePath("debuggable-app")));
-    if (nm.waitForStarted()) { // Let's ignore hosts without nm.
-        QVERIFY2(nm.waitForFinished(), qPrintable(nm.errorString()));
-        QVERIFY2(nm.exitCode() == 0, nm.readAllStandardError().constData());
-        const QByteArray output = nm.readAllStandardOutput();
-        QVERIFY2(output.toLower().contains("debugginghelper"), output.constData());
-    }
+    nm.start("nm", QStringList(relativeExecutableFilePath("debuggable-app", m_qbsStdout)));
+    if (!nm.waitForStarted())
+        QSKIP("The remainder of this test requires nm");
+
+    QVERIFY2(nm.waitForFinished(), qPrintable(nm.errorString()));
+    QVERIFY2(nm.exitCode() == 0, nm.readAllStandardError().constData());
+    const QByteArray output = nm.readAllStandardOutput();
+    QVERIFY2(output.toLower().contains("debugginghelper"), output.constData());
 }
 
 void TestBlackboxQt::qobjectInObjectiveCpp()
@@ -654,10 +857,12 @@ void TestBlackboxQt::track_qobject_change()
     QDir::setCurrent(testDataDir + "/trackQObjChange");
     copyFileAndUpdateTimestamp("bla_qobject.h", "bla.h");
     QCOMPARE(runQbs(), 0);
-    const QString productFilePath = relativeExecutableFilePath("i");
+
+    const QString objectSuffix = parsedObjectSuffix(m_qbsStdout);
+    const QString productFilePath = relativeExecutableFilePath("i", m_qbsStdout);
     QVERIFY2(regularFileExists(productFilePath), qPrintable(productFilePath));
-    QString moc_bla_objectFileName = relativeProductBuildDir("i") + '/'
-            + inputDirHash("qt.headers") + objectFileName("/moc_bla.cpp", profileName());
+    QString moc_bla_objectFileName = relativeProductBuildDir("i") + '/' + inputDirHash("qt.headers")
+                                     + "/moc_bla.cpp" + objectSuffix;
     QVERIFY2(regularFileExists(moc_bla_objectFileName), qPrintable(moc_bla_objectFileName));
 
     WAIT_FOR_NEW_TIMESTAMP();
@@ -675,10 +880,10 @@ void TestBlackboxQt::track_qrc()
         QSKIP("Qt version too old");
     if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
         QSKIP("Cannot run binaries in cross-compiled build");
+    const QString fileName = relativeExecutableFilePath("i", m_qbsStdout);
     QCOMPARE(runQbs(QbsRunParameters("run")), 0);
     QVERIFY2(m_qbsStdout.contains("rcc"), m_qbsStdout.constData());
     QVERIFY2(!m_qbsStdout.contains("compiling test.cpp"), m_qbsStdout.constData());
-    const QString fileName = relativeExecutableFilePath("i");
     QVERIFY2(regularFileExists(fileName), qPrintable(fileName));
 
     QDateTime dt = QFileInfo(fileName).lastModified();
